/////////////////////////////////////////////////////////////////////////////////////////////////////
// Modifications in "png.c":
/////////////////////////////////////////////////////////////////////////////////////////////////////

//>>>sTER
PNG_sTER;
//<<<sTER

// End of Modifications in "png.c".
/////////////////////////////////////////////////////////////////////////////////////////////////////




/////////////////////////////////////////////////////////////////////////////////////////////////////
// Modifications in "png.h":
/////////////////////////////////////////////////////////////////////////////////////////////////////

//>>>sTER
 * Contributing Author for Stereoscopic support
 * 
 *    Jean-Claude Hbert
 *
//<<<sTER

//>>>sTER
#if defined(PNG_sTER_SUPPORTED)
   /* It is the presence of the sTER chunk that indicates that the image is
	 * stereoscopic.  The data portion of the stereoscopic sTER chunk consists
	 * of an unsigned 8 bits integer that contains the stereoscopic image type.
	 * Up to now, since no type value has been yet defined, the stereoscopic
	 * image type value must always be initialized to 0.  However, numbers
	 * greater than 127 (128 to 255) are allowed for inventing experimental or
	 * private definitions of values.  (Note that use of private type codes may
	 * render a file unreadable by standard decoders.  Such codes are strongly
	 * discouraged except for experimental purposes, and should not appear in
	 * publicly available software or files.)  It is more the way that the left
	 * and right images are stored than the chunk itself that makes the .PNG image
	 * a stereoscopic image. 
	 *
	 * The rules are very easy.  The .PNG image is composed of two images.
	 * One for the right eye and one for the left eye.  Right and left images
	 * must have the same size.  Samples of right and left images are co-sited.
	 * The two images are stored side by side horizontally with sometimes some
	 * padding pixels between them.  Right image come first (right image is at
	 * left of left image).  The horizontal start position of left image must
	 * always begin on a multiple of 8 pixels (in order to sync the Progressive
	 * Display of the two images).  You must use the pHYs chunk to indicate the
	 * pixels size ratio when it is not 1:1.
	 *
	 * An example will help understanding the way a stereoscopic image of 77x53
	 * pixels is saved like a normal image of 157x53 pixels.
	 *  - Right and left images are 77x53 pixels.
	 *  - Rectangle (  0,0)->( 76,52) inclusively contains the right image.
	 *  - Rectangle (77,0)->( 79,52) inclusively contains the padding pixels.
	 *  - Rectangle (80,0)->(156,52) inclusively contains the left image.
	 *  - So the total size of the saved .PNG image is 157x53 pixels.
	 *
	 * IMPORTANT: Even if the padding pixels are never displayed, their values
	 * must always be valid because an application, not supporting the sTER
	 * chunk, will display the complete image with the padding pixels.
	 * (The only dangerous case is the color palette case with less entries in
	 * the palette than the bit depth can produces.)  In the example above, we
	 * are using grayscale padding pixels only to make the padding region more
	 * obvious to see.
	 *
	 * ENCODER EXAMPLE:
	 * We take right and left images from an ALIEN Video Camera.  The sizes of
	 * each mono images are 77x53 pixels.  The height of an ALIEN pixel is
	 * around 20% taller than its width so a ratio of 12:10 (HzPix : VtPix)
	 * will be used.  We call the png_get_sTER_total_and_padding_width()
	 * function to calculate the composed image width.
	 * We merge right and left images into the composed image of 157x53 pixels
	 * (157 = 77+3+77; 3 = padding pixels width).  We set the pHYs chunk to
	 * {12, 10, 0} (0 because the pixel physical size is unknown in this
	 * example).  We save the composed image with the sTER and pHYs chunks.
	 *
	 * DECODER EXAMPLE:
	 * In this example we will read the Stereo PNG file created in the above
	 * example.  A typical web browser decoder gets a size of 157x53 pixels
	 * from the PNG header.  It finds a sTER chunk so it calls the
	 * png_get_sTER_real_and_padding_width() function to calculate the stereo
	 * image width, so a stereo image size of 77x53 pixels is expected. It finds
	 * the pHYs chunk with a ratio of 12:10, so it reserves a rectangle area of
	 * 77x64 pixels (64 = 12/10*53) on the displayed webpage to draw the stereo
	 * image (assuming screen pixels are square).  The progressive reading is
	 * going on (if the stereo image was saved with the Adam7 feature) and at
	 * some interval of time, the stereo displaying engine extracts the right
	 * and left images from the partially loaded 157x53 pixels composed image,
	 * it creates the resulting stereo image by merging the right and left
	 * images together using the current browser stereo displaying method, that
	 * has been specified by the user preferences, and finally it stretches the
	 * resulting stereo image to fill the reserved rectangle area of 77x64
	 * pixels on the webpage.  The final result is a well proportioned stereo
	 * image identical to the one taken from the ALIEN Video Camera.  Of course
	 * if the HTML code specifies directly the stereo image size, the stereo
	 * displaying engine will use that size to reserved the rectangle area on
	 * the webpage.  If a size is specified, it is because the HTML author is
	 * knowing what he does.
	 *
    * 'stereo_type' value is valid only if (valid & PNG_INFO_sTER) is non-zero.
    */
   png_byte stereo_type;  /* reserved for future used */
#endif /* PNG_sTER_SUPPORTED */
//<<<sTER

//>>>sTER
#define PNG_STEREO_TYPE_NONE		 0	/* No type value defined up to now.  Reserved for future use. */
#define PNG_STEREO_TYPE_LAST      1	/* Not a valid value */
//<<<sTER

//>>>sTER
#define PNG_INFO_sTER 0x00010000
//<<<sTER

//>>>sTER
/* Returns image stereo_type, from sTER chunk data. */
extern PNG_EXPORT(png_byte, png_get_stereo_type) PNGARG((png_structp
png_ptr, png_infop info_ptr));
//<<<sTER
					
//>>>sTER
#if defined(PNG_sTER_SUPPORTED)
extern PNG_EXPORT(png_uint_32,png_get_sTER) PNGARG((png_structp png_ptr,
   png_infop info_ptr, png_byte *stereo_type));
extern PNG_EXPORT(png_uint_32,png_get_sTER_real_and_padding_width) PNGARG(
   (png_uint_32 total_width, png_uint_32 *real_width, png_uint_32 *padding_width));
extern PNG_EXPORT(void,png_get_sTER_total_and_padding_width) PNGARG(
   (png_uint_32 real_width, png_uint_32 *total_width, png_uint_32 *padding_width));
#endif

#if defined(PNG_sTER_SUPPORTED)
extern PNG_EXPORT(void,png_set_sTER) PNGARG((png_structp png_ptr,
   png_infop info_ptr, png_byte stereo_type));
#endif
//<<<sTER

//>>>sTER
#define PNG_stER const png_byte png_sTER[5] = {115, 116,  69,  82, '\0'}
#define PNG_sTER PNG_stER		// Until sTER is recognized as a public chunk.
//#define PNG_sTER const png_byte png_sTER[5] = {115,  84,  69,  82, '\0'}
//<<<sTER

//>>>sTER
PNG_EXPORT_VAR (const png_byte FARDATA) png_sTER[5];
//<<<sTER

//>>>sTER
#if defined(PNG_WRITE_sTER_SUPPORTED)
PNG_EXTERN void png_write_sTER PNGARG((png_structp png_ptr,
	png_byte stereo_type));
#endif
//<<<sTER

//>>>sTER
#if defined(PNG_READ_sTER_SUPPORTED)
PNG_EXTERN void png_handle_sTER PNGARG((png_structp png_ptr, png_infop info_ptr,
   png_uint_32 length));
#endif
//<<<sTER

// End of Modifications in "png.h".
/////////////////////////////////////////////////////////////////////////////////////////////////////




/////////////////////////////////////////////////////////////////////////////////////////////////////
// Modifications in "pngconf.h":
/////////////////////////////////////////////////////////////////////////////////////////////////////

//>>>sTER
#  define PNG_NO_READ_sTER
#  define PNG_NO_WRITE_sTER
//<<<sTER

//>>>sTER
 * png_get_stereo_type()
//<<<sTER

//>>>sTER
#ifndef PNG_NO_READ_sTER
#  define PNG_READ_sTER_SUPPORTED
#  define PNG_sTER_SUPPORTED
#endif
//<<<sTER

//>>>sTER
#ifndef PNG_NO_WRITE_sTER
#  define PNG_WRITE_sTER_SUPPORTED
#  ifndef PNG_sTER_SUPPORTED
#    define PNG_sTER_SUPPORTED
#  endif
#endif
//<<<sTER

// End of Modifications in "pngconf.h".
/////////////////////////////////////////////////////////////////////////////////////////////////////




/////////////////////////////////////////////////////////////////////////////////////////////////////
// Modifications in "pngget.c":
/////////////////////////////////////////////////////////////////////////////////////////////////////

//>>>sTER
png_byte	PNGAPI
png_get_stereo_type(png_structp png_ptr, png_infop info_ptr)
{
   if (png_ptr != NULL && info_ptr != NULL)
#if defined(PNG_sTER_SUPPORTED)
   if (info_ptr->valid & PNG_INFO_sTER)
   {
      png_debug1(1, "in %s retrieval function\n", "png_get_stereo_type");
      return (info_ptr->stereo_type);
   }
#else
   return (0);
#endif
   return (0);
}
//<<<sTER

//>>>sTER
#if defined(PNG_sTER_SUPPORTED)
png_uint_32 PNGAPI
png_get_sTER(png_structp png_ptr, png_infop info_ptr, png_byte *stereo_type)
{
   if (png_ptr != NULL && info_ptr != NULL && info_ptr->valid & PNG_INFO_sTER &&
       stereo_type != NULL)
   {
      png_debug1(1, "in %s retrieval function\n", "sTER");
      *stereo_type = info_ptr->stereo_type;
      return (PNG_INFO_sTER);
   }
   return (0);
}
#endif

#if defined(PNG_sTER_SUPPORTED)
/* Caculate from "real_width" (stereo image width) the
 * "padding_width" (padding pixels width between the
 * right and left images) and the "total_width" (width
 * that will be written in the header chunk).
 * Note: The "total_width" consists of the "real_width" of the
 *       right image + the "padding_width" (to round up to the
 *       next multiple of 8) + the "real_width" of the left image.
 *       ("real_width" must always be greater than zero.)
 *       (Left and right images size must be always equal.)
 */
void PNGAPI
png_get_sTER_total_and_padding_width(png_uint_32 real_width,
   png_uint_32 *total_width, png_uint_32 *padding_width)
{
	png_uint_32 total_w, padding_w;

	if ( (padding_w = (8 - (real_width % 8))) == 8 ) padding_w = 0;
	
	total_w = ((real_width * 2) + padding_w);

	if ( total_width   != NULL ) *total_width   = total_w;
	if ( padding_width != NULL ) *padding_width = padding_w;
}
#endif

#if defined(PNG_sTER_SUPPORTED)
/* Caculate from "total_width" (width written in the header chunk)
 * the "padding_width" (padding pixels width between the right and
 * left images) and the "real_width" (stereo image width).
 * Note: The "total_width" consists of the "real_width" of the right
 *       image + the "padding_width" (to round up to the next
 *       multiple of 8) + the "real_width" of the left image.
 *       ("real_width" must always be greater than zero.)
 *       (Left and right images size must be always equal.)
 * Result: If "total_width" is a Valid Stereo Image width then
 *         the caculated values are stored in "padding_width" and
 *         "real_width" and a non-zero value is returned.
 *         If "total_width" is NOT a Valid Stereo Image width then
 *         "padding_width" and "real_width" are set to zero and
 *          zero is returned.
 * IMPORTANT: When saving a Stereo Image, always use the
 *            "png_get_sTER_total_and_padding_width()"
 *            function to obtain a Valid Stereo Image width
 *            that will be written in the header chunk.
 */
png_uint_32	PNGAPI
png_get_sTER_real_and_padding_width(png_uint_32 total_width,
   png_uint_32 *real_width, png_uint_32 *padding_width)
{
	png_uint_32 real_w, padding_w;
	png_uint_32 stereo_status = 0;

	if ( total_width >= 9 ) /* Maybe Valid Stereo Image? */
	{
		if ( (padding_w = (8 - (total_width % 8))) == 8 ) padding_w = 0;
		
		real_w = ((total_width - padding_w) / 2);
		
		if ( ((real_w + padding_w) % 8) == 0 ) /* Valid Stereo Image? */
		{
			stereo_status = 1;
		}
	}

	if ( stereo_status == 0 ) /* Invalid Stereo Image? */
	{
		padding_w = real_w = 0;
	}

	if ( real_width    != NULL ) *real_width    = real_w;
	if ( padding_width != NULL ) *padding_width = padding_w;

	return( stereo_status );
}
#endif
//<<<sTER

// End of Modifications in "pngget.c".
/////////////////////////////////////////////////////////////////////////////////////////////////////




/////////////////////////////////////////////////////////////////////////////////////////////////////
// Modifications in "pngpread.c":
/////////////////////////////////////////////////////////////////////////////////////////////////////

//>>>sTER
#if defined(PNG_READ_sTER_SUPPORTED)
      PNG_sTER;
#endif
//<<<sTER

//>>>sTER
#if defined(PNG_READ_sTER_SUPPORTED)
   else if (!png_memcmp(png_ptr->chunk_name, png_sTER, 4))
   {
      if (png_ptr->push_length + 4 > png_ptr->buffer_size)
      {
         png_push_save_buffer(png_ptr);
         return;
      }
      png_handle_sTER(png_ptr, info_ptr, png_ptr->push_length);
   }
#endif
//<<<sTER

// End of Modifications in "pngpread.c".
/////////////////////////////////////////////////////////////////////////////////////////////////////




/////////////////////////////////////////////////////////////////////////////////////////////////////
// Modifications in "pngread.c":
/////////////////////////////////////////////////////////////////////////////////////////////////////

//>>>sTER
#if defined(PNG_READ_sTER_SUPPORTED)
      PNG_sTER;
#endif
//<<<sTER

//>>>sTER
#if defined(PNG_READ_sTER_SUPPORTED)
      else if (!png_memcmp(png_ptr->chunk_name, png_sTER, 4))
         png_handle_sTER(png_ptr, info_ptr, length);
#endif
//<<<sTER

//>>>sTER
#if defined(PNG_READ_sTER_SUPPORTED)
      PNG_sTER;
#endif
//<<<sTER

//>>>sTER
#if defined(PNG_READ_sTER_SUPPORTED)
      else if (!png_memcmp(png_ptr->chunk_name, png_sTER, 4))
         png_handle_sTER(png_ptr, info_ptr, length);
#endif
//<<<sTER

// End of Modifications in "pngread.c".
/////////////////////////////////////////////////////////////////////////////////////////////////////




/////////////////////////////////////////////////////////////////////////////////////////////////////
// Modifications in "pngrutil.c":
/////////////////////////////////////////////////////////////////////////////////////////////////////

//>>>sTER
#if defined(PNG_READ_sTER_SUPPORTED)
void /* PRIVATE */
png_handle_sTER(png_structp png_ptr, png_infop info_ptr, png_uint_32 length)
{
   png_byte buf[1];
	png_byte stereo_type;

   png_debug(1, "in png_handle_sTER\n");

   if (!(png_ptr->mode & PNG_HAVE_IHDR))
      png_error(png_ptr, "Missing IHDR before sTER");
   else if (png_ptr->mode & PNG_HAVE_IDAT)
   {
      png_warning(png_ptr, "Invalid sTER after IDAT");
      png_crc_finish(png_ptr, length);
      return;
   }
   else if (info_ptr != NULL && (info_ptr->valid & PNG_INFO_sTER))
   {
      png_warning(png_ptr, "Duplicate sTER chunk");
      png_crc_finish(png_ptr, length);
      return;
   }

   if (length != 1)
   {
      png_warning(png_ptr, "Incorrect sTER chunk length");
      png_crc_finish(png_ptr, length);
      return;
   }

   png_crc_read(png_ptr, buf, 1);
   if (png_crc_finish(png_ptr, 0))
      return;

   stereo_type = buf[0];
   png_set_sTER(png_ptr, info_ptr, stereo_type);
}
#endif
//<<<sTER

// End of Modifications in "pngrutil.c".
/////////////////////////////////////////////////////////////////////////////////////////////////////




/////////////////////////////////////////////////////////////////////////////////////////////////////
// Modifications in "pngset.c":
/////////////////////////////////////////////////////////////////////////////////////////////////////

//>>>sTER
#if defined(PNG_sTER_SUPPORTED)
void PNGAPI
png_set_sTER(png_structp png_ptr, png_infop info_ptr,
   png_byte stereo_type)
{
   png_debug1(1, "in %s storage function\n", "sTER");
   if (png_ptr == NULL || info_ptr == NULL)
      return;

   info_ptr->stereo_type = stereo_type;
   info_ptr->valid |= PNG_INFO_sTER;
}
#endif
//<<<sTER

// End of Modifications in "pngset.c".
/////////////////////////////////////////////////////////////////////////////////////////////////////




/////////////////////////////////////////////////////////////////////////////////////////////////////
// Modifications in "pngwrite.c":
/////////////////////////////////////////////////////////////////////////////////////////////////////

//>>>sTER
#if defined(PNG_WRITE_sTER_SUPPORTED)
   if (info_ptr->valid & PNG_INFO_sTER)
      png_write_sTER(png_ptr, info_ptr->stereo_type);
#endif
//<<<sTER

// End of Modifications in "pngwrite.c".
/////////////////////////////////////////////////////////////////////////////////////////////////////




/////////////////////////////////////////////////////////////////////////////////////////////////////
// Modifications in "pngwutil.c":
/////////////////////////////////////////////////////////////////////////////////////////////////////

//>>>sTER
#if defined(PNG_WRITE_sTER_SUPPORTED)
/* write the sTER chunk */
void /* PRIVATE */
png_write_sTER(png_structp png_ptr, png_byte stereo_type)
{
#ifdef PNG_USE_LOCAL_ARRAYS
   PNG_sTER;
#endif
   png_byte buf[1];

   png_debug(1, "in png_write_sTER\n");
   if (stereo_type >= PNG_STEREO_TYPE_LAST)
      png_warning(png_ptr, "Unrecognized type for sTER chunk");

   buf[0] = stereo_type;

   png_write_chunk(png_ptr, (png_bytep)png_sTER, buf, (png_size_t)1);
}
#endif
//<<<sTER

// End of Modifications in "pngwutil.c".
/////////////////////////////////////////////////////////////////////////////////////////////////////
